@@ -1,6 +1,3 @@
-
-OPAM_PKGS="oasis base-bytes"
-
 export OPAMYES=1
 
 if [ -f "$HOME/.opam/config" ]; then
@@ -15,8 +12,8 @@ if [ -n "${OPAM_SWITCH}" ]; then
 fi
 eval `opam config env`
 
-opam install $OPAM_PKGS
-
 export OCAMLRUNPARAM=b
-make
-make test
+
+opam pin -y add csv .
+opam pin -y add csv-lwt .
+
