@@ -26,6 +26,7 @@
     {{:https://tools.ietf.org/html/rfc4180} RFC4180} if one sets
     [strip=false] in the creation functions.
 
+    @version %%VERSION%%
     @author Richard Jones <rjones\@redhat.com>
     @author Christophe Troestler <Christophe.Troestler\@umons.ac.be>
  *)
@@ -85,9 +86,14 @@ type in_channel
 val of_in_obj : ?separator:char -> ?strip: bool ->
                 ?has_header: bool -> ?header: string list ->
                 ?backslash_escape: bool -> ?excel_tricks:bool ->
+                ?fix:bool ->
                 in_obj_channel -> in_channel
 (** [of_in_obj ?separator ?excel_tricks in_chan] creates a new "channel"
     to access the data in CSV form available from the channel [in_chan].
+    Note that data is read on a as-needed basis by the functions {!next},
+    {!fold_left},... so the channel must stay open while these functions
+    are executed.  When you are done with the channel, call {!close_in} to
+    close it.
 
     @param separator What character the separator is.  The default is
     [','].  You should be aware however that, in the countries where
@@ -116,24 +122,33 @@ val of_in_obj : ?separator:char -> ?strip: bool ->
     followed by '0' in a quoted string means ASCII NULL and the fact
     that a field of the form ="..." only returns the string inside the
     quotes.  Default: [true].
- *)
+
+    @param fix Parses the CSV data without raising the exception
+    [Csv.Failure].  If the data does not conform to the CSV format
+    (e.g. because of badly escaped quotes), try to repair it.
+    Default: [false].  *)
 
 val of_channel : ?separator:char -> ?strip: bool ->
                  ?has_header: bool -> ?header: string list ->
                  ?backslash_escape: bool -> ?excel_tricks:bool ->
+                 ?fix: bool ->
                  Pervasives.in_channel -> in_channel
   (** Same as {!Csv.of_in_obj} except that the data is read from a
-      standard channel. *)
+      standard channel.
+      Note that, because the data is read on a as-needed basis, the
+      [Pervasive.in_channel] must stay open while you are reading data. *)
 
 val of_string : ?separator:char -> ?strip: bool ->
                 ?has_header: bool -> ?header: string list ->
                 ?backslash_escape: bool -> ?excel_tricks:bool ->
+                ?fix: bool ->
                 string -> in_channel
   (** Same as {!Csv.of_in_obj} except that the data is read from a
       string. *)
 
 val load : ?separator:char -> ?strip: bool ->
-           ?backslash_escape: bool -> ?excel_tricks:bool-> string -> t
+           ?backslash_escape: bool -> ?excel_tricks:bool -> ?fix:bool ->
+           string -> t
   (** [load fname] loads the CSV file [fname].  If [filename] is ["-"]
       then load from [stdin].
 
@@ -149,10 +164,13 @@ val load : ?separator:char -> ?strip: bool ->
       @param excel_tricks enables Excel tricks, namely the fact that '"'
       followed by '0' in a quoted string means ASCII NULL and the fact
       that a field of the form ="..." only returns the string inside the
-      quotes.  Default: [true].  *)
+      quotes.  Default: [true].
+
+      @param fix Fix invalid CSV in order to parse it without raising
+      [Csv.Failure].  See {!of_in_obj}. *)
 
 val load_in : ?separator:char -> ?strip: bool ->
-              ?backslash_escape: bool -> ?excel_tricks:bool ->
+              ?backslash_escape: bool -> ?excel_tricks:bool -> ?fix: bool ->
               Pervasives.in_channel -> t
   (** [load_in ch] loads a CSV file from the input channel [ch].
       See {!Csv.load} for the meaning of [separator] and [excel_tricks]. *)
@@ -176,7 +194,7 @@ val next : in_channel -> string list
       @raise End_of_file if no more record can be read.
 
       @raise Csv.Failure if the CSV format is not respected.  The
-      partial record read is available with [#current_record]. *)
+      partial record read is available with {!Csv.current_record}. *)
 
 val fold_left : f:('a -> string list -> 'a) -> init:'a -> in_channel -> 'a
   (** [fold_left f a ic] computes (f ... (f (f a r0) r1) ... rN)
@@ -206,6 +224,7 @@ val current_record : in_channel -> string list
 
 val load_rows : ?separator:char -> ?strip: bool ->
                 ?backslash_escape: bool -> ?excel_tricks:bool ->
+                ?fix: bool ->
                 (string list -> unit) -> Pervasives.in_channel -> unit
   (** @deprecated use {!Csv.iter} on a {!Csv.in_channel} created with
       {!Csv.of_channel}. *)
@@ -217,6 +236,7 @@ type out_channel
 
 val to_out_obj : ?separator:char ->
                  ?backslash_escape: bool -> ?excel_tricks:bool ->
+                 ?quote_all:bool ->
                  out_obj_channel -> out_channel
   (** [to_out_obj ?separator ?excel_tricks out_chan] creates a new "channel"
       to output the data in CSV form.
@@ -233,15 +253,20 @@ val to_out_obj : ?separator:char ->
       '\000' is represented as '"' followed by '0' and the fact that a
       field with leading or trailing spaces or a leading '0' will be
       encoded as ="..."  (to avoid Excel "helping" you).  Default:
-      [false].  *)
+      [false].
+
+      @param quote_all force all fields to be quoted, even if this
+      is not required by the CSV specification. *)
 
 val to_channel : ?separator:char ->
                  ?backslash_escape: bool -> ?excel_tricks:bool ->
+                 ?quote_all:bool ->
                  Pervasives.out_channel -> out_channel
   (** Same as {!Csv.to_out_obj} but output to a standard channel. *)
 
 val to_buffer : ?separator:char ->
                 ?backslash_escape: bool -> ?excel_tricks:bool ->
+                ?quote_all:bool ->
                 Buffer.t -> out_channel
   (** Same as {!Csv.to_out_obj} but output to a buffer. *)
 
@@ -263,10 +288,12 @@ val save_out : ?separator:char ->
   (** @deprecated Save string list list to a channel. *)
 
 val save : ?separator:char -> ?backslash_escape: bool -> ?excel_tricks:bool ->
+           ?quote_all:bool ->
            string -> t -> unit
   (** [save fname csv] saves the [csv] data to the file [fname]. *)
 
 val print : ?separator:char -> ?backslash_escape: bool -> ?excel_tricks:bool ->
+            ?quote_all:bool ->
             t -> unit
   (** Print the CSV data. *)
 
@@ -287,7 +314,7 @@ val save_out_readable : Pervasives.out_channel -> t -> unit
 
 (** A row with a header. *)
 module Row : sig
-  type t
+  type t = Csv_row.t
   (** Representation of a row whose columns are accessible both by
       indices and by headers names. *)
 
@@ -299,7 +326,7 @@ module Row : sig
 
   val find : t -> string -> string
   (** [find row header] return the value of the colum labelled with
-      [header] (or [""] if not such header has been declared).  *)
+      [header] (or [""] if no such header has been declared).  *)
 
   val to_list : t -> string list
   (** [to_list row] convert [row] to the usual representation, the
@@ -321,6 +348,17 @@ module Rows : sig
   val header : in_channel -> string list
   (** The header declared for this channel.  *)
 
+  val set_header : ?replace: bool -> in_channel -> string list -> unit
+  (** [set_header ic headers] set the new header for all {i
+     subsequent} reads on [ic] (previously read {!Row.t} values are
+     unaffected).  This is useful for example to compose an unusual
+     header after reading several lines of the CSV data.
+
+     @param replace When [false] (the default), if an entry in
+     [headers] is [""], the previous name (if any) is kept (see the
+     [header] parameter of {!of_in_obj}).  When [true], an entry [""]
+     means there is no access by name for that column. *)
+
   val next : in_channel -> Row.t
   (** See {!Csv.next}.  If no header was declared for the channel,
       this function will work but only access using {!Row.get} will
@@ -341,8 +379,12 @@ module Rows : sig
   val load : ?separator:char -> ?strip: bool ->
              ?has_header: bool -> ?header: string list ->
              ?backslash_escape: bool -> ?excel_tricks:bool ->
+             ?fix: bool ->
              string -> Row.t list
-  (** See {!Csv.load}. *)
+  (** See {!Csv.load} and {!Csv.of_in_obj} for the optional parameters.
+     Note that [has_header] is false by default to have a uniform
+     interface but you likely want to set it to [true] or to
+     explicitly provide [header].  *)
 
   val current : in_channel -> Row.t
   (** See {!Csv.current_record}. *)
